function [h] = plot_u1_deformation(PD, scaleFactor, indx)

h_c = PD.wavelength_c;
w_c = 2*pi/h_c;
x_min = 0;
x_max = h_c;
y_min_1 = 0;
y_max_1 = PD.L1;

y_min_2 = PD.L1;
y_max_2 = 1.0;

x = linspace(x_min, x_max, 300);
y_1 = linspace(y_min_1, y_max_1, 270);
[X_1, Y_1] = meshgrid(x, y_1);
Z_1 = zeros(size(X_1));

[X_1_new, Y_1_new, u1_1] = deformMesh(PD, scaleFactor, X_1, Y_1);

y_2 = linspace(y_min_2, y_max_2, 30);
[X_2, Y_2] = meshgrid(x, y_2);
Z_2 = zeros(size(X_2)) + 1;

[X_2_new, Y_2_new, u1_2] = deformMesh(PD, scaleFactor, X_2, Y_2);


h = figure(1);
s1 = surf(X_1_new, Y_1_new, Z_1);
s1.EdgeColor = 'none';
hold on;
s2 = surf(X_2_new, Y_2_new, Z_2);
s2.EdgeColor = 'none';
grid off;
colormap jet;
set(groot,'defaultLegendInterpreter','latex');

view(2)
axis equal;
xlim([0 1.8])

fileName = ['deformation_', num2str(indx),  '.pdf'];
set(gcf, 'Color', 'None');
set(gca, 'Color', 'None');
set(gca,'visible','off')
axis([0, 1.5, 0, 1.5]);
export_fig(fileName, '-pdf', '-transparent')


h = figure(2);
s3 = surf(X_1, Y_1, u1_1);
s3.EdgeColor = 'none';
hold on;
s4 = surf(X_2, Y_2, u1_2);
s4.EdgeColor = 'none';
grid on




%% Try another way

x = linspace(0, h_c, 1000);
X_vals = [x; x; x]; 
Y_vals = [zeros(size(x)); PD.L1*ones(size(x)); ones(size(x))];
[X_vals, Y_vals, ~] = deformMesh(PD, scaleFactor, X_vals, Y_vals);

close all;
figure(3)
hold on;
c = jet;
a1 = area(X_vals(3, :), Y_vals(3, :))
a1(1).FaceColor = c(end, :);
a2 = area(X_vals(2, :), Y_vals(2, :))
a2(1).FaceColor = c(1, :);

fileName = ['deformation_line_', num2str(indx),  '.pdf'];
set(gcf, 'Color', 'None');
set(gca, 'Color', 'None');
set(gca,'visible','off')
axis equal;
xlim([0, 1.8]);
figure(3)
export_fig(fileName, '-pdf', '-transparent')

end


function [X_new, Y_new, Z_new] = deformMesh(PD, scaleFactor, X, Y)
h_c = PD.wavelength_c;
w_c = 2*pi/h_c;
A = PD.A;
B = PD.B;
a = PD.alphas;

X_new = zeros(size(X));
Y_new = zeros(size(Y));
Z_new = X_new;

lambda1 = 1 - PD.lambda_c;
lambda2 = get_lambda_2(PD.lambda_c);
for i = 1:size(X, 1)
    for j = 1:size(X,2)
        
        x1 = X(i,j);
        x2 = Y(i,j);
        v1 = 0;
        v2 = 0;
        t = 0;
        if (x2 >= PD.L1)
            t = 4;
        end
        for m = 1:4
           v1 = v1 + A(m + t)*exp(a(m)*x2); 
           v2 = v2 + B(m)*A(m + t)*exp(a(m)*x2); 
        end

        u11 = -(v1).*(sin(w_c*x1));
        u12 = (v2).*(cos(w_c*x1));
        
        u01 = (lambda1 - 1)*x1;
        u02 = (lambda2 - 1)*x2;
        X_new(i,j) = X(i,j) + scaleFactor*u11 + u01;
        Y_new(i,j) = Y(i,j) + scaleFactor*u12 + u02;
        
        Z_new(i,j) = scaleFactor*u12;
                
    end
end



end

