clear
clc
close all


% This is a script that will generate the plots with transparency. It has
% to make two plots: one for the axis and another for the transparent
% surface. These will be named 'example_figure.pdf' and 
% 'example_figure.svg', respectivly. They will have a transparent
% background, and be ready to be imported in your favorite image processing
% software to cook together.
% Files Included: 
% example_plot.m       : This file.
% data.mat             : The test data that we will be plotting.
% 'example_cooked.svg' : An example figure after we have combined the two
%                        outputted images.
% include/*            : All the utilites needed for plotting.
% By : Andrew Akerson, Feb 18 2020.

% first we need to add the figure exporting utilities to the path
addpath(genpath('./include'))

% Load the data. This loads in the following data:
% X : 80x80 double
% Y : 80x80 double
% Z : 80x80 double
% (X and Y were generated with a meshgrid call, and Z is the computed 
% values from those)
load('data.mat')

figure(1);

% these are just some things that give nice latex font for the axis and
% labels. You don't need these if you dont want.
set(0,'defaulttextinterpreter','latex');
set(groot,'defaultAxesTickLabelInterpreter','latex');
set(groot,'defaulttextinterpreter','latex');
set(groot,'defaultLegendInterpreter','latex');

% make the surface
S1 = surf(X, Y, Z);

% I chose colormap jet because it has good contrast. You can change this to
% some other if you want. 
colormap jet

% Here I fuck around with the font sizes. Again, this is optional
ax = gca;
ax.YAxis.FontSize = 12;
ax.XAxis.FontSize = 12;
ax.ZAxis.FontSize = 12;

% I label these, and use latex interpreter just in case greek letters are
% on the menue. Here it is just some normal letters.
ylabel('$Y$',...
    'interpreter','latex','fontsize',18)
xlabel('$X$        ',...
    'Interpreter','latex','FontSize',18)
zlabel('$Z$        ',...
    'Interpreter','latex','FontSize',18, 'rot', 1, 'VerticalAlignment','middle', 'HorizontalAlignment', 'right')

% get a good view to properly visualize the data. This was obtained from
% trial and error, and would depend on the data you have.
view(-25, 35);

% gotta name that output file
fileName = 'example_figure.pdf';

% get rid of all the background colors and stuff (so the background is
% transparent)
set(gcf, 'Color', 'None');
set(gca, 'Color', 'None');

% Here I just set axis limits and tick values. This will change depending
% on the data you have.
xlim([1, 60]);
xticks([1 10 20 30 40 50 60]);

ylim([0, 0.6]);
yticks([0 0.2 0.4 0.6]);
zticks(0:0.1:0.5);

% I don't really remember why I am doing this. If I were to guess, I am
% getting the axis limits, getting rid of the axis, but resetting the
% limits somehow? not really sure lol. For some reason this was there tho.
XL = get(gca, 'XLim');
YL = get(gca, 'YLim');
ZL = get(gca, 'ZLim');
set(S1, 'visible', 'off');
axis([XL, YL, ZL]);

% This is where I do the exportfig call. This is the one that will have the
% axis and the ticks and stuff, but not the surface.
export_fig(fileName, '-pdf', '-transparent')

% Now here is where we export the svg that will have the transparency
% plot, but not the axis. S1.EdgeAlpha and S1.FaceAlpha will change how
% transparent it is. Right now it is set to 50%.
set(S1, 'Visible', 'on')
S1.EdgeAlpha = 0.5;
S1.FaceAlpha = 0.5;
fig2svg([fileName(1:end-3), 'svg'], figure(1), 'png')
